package es.um.atica.unica2.backbeans;

import java.util.HashMap;

import org.jboss.seam.Component;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.log.Log;
import org.primefaces.model.menu.DefaultMenuItem;
import org.primefaces.model.menu.DefaultMenuModel;
import org.primefaces.model.menu.MenuModel;

/**
 * Define el controlador del Wizard. Gestiona el cambio entre pantallas repintado de botones y acciones posibles a
 * realizar
 * 
 * @author pedrody
 */
@Name( "manejadorWizard" )
@Scope( ScopeType.CONVERSATION )
public class ManejadorWizard {

    @Logger
    private Log log;

    /**
     * Este enumerado define el numero de pantallas que tiene el wizard. El orden en el que se definan los enumerados
     * establece el orden de las pantallas
     */
    public enum PasoWizard {
        // TODO: Completar con los valores del enumerado con NOMBREENUMERADO
        // ("Texto a aparecer en los pasos del wizard")
        // Ejemplo: EXPEDIENTES( "Expedientes" ), DATOS_EXP( "Datos del expediente" ) .....
        PAGINA1( "pagina1" ), PAGINA2( "pagina2" ), PAGINA3( "pagina3" );

        private String descripcion;

        private PasoWizard( String descripcion ) {
            this.descripcion = descripcion;
        }

        public String getDescripcion() {
            return descripcion;
        }

        public void setDescripcion( String descripcion ) {
            this.descripcion = descripcion;
        }

    }

    private MenuModel menu;

    private HashMap<PasoWizard, String> manejadores;

    // Define el paso (pantalla) en la que nos encontramos
    // Se debe inicializar en el @Create
    public PasoWizard pasoActual;

    @Create
    public void inicializaWizard() {
        // Se situa en el primer paso
        pasoActual = PasoWizard.PAGINA1;

        // Carga los manejadores de cada pantalla del wizard.
        // Se introduce para cada paso el identificador del manejador para cargarlo dinamicamente
        manejadores = new HashMap<ManejadorWizard.PasoWizard, String>();
        manejadores.put( PasoWizard.PAGINA1, "manejadorPagina1" );
        manejadores.put( PasoWizard.PAGINA2, "manejadorPagina2" );

        // Inicializa el menu de pasos del wizard
        menu = new DefaultMenuModel();
        for ( PasoWizard paso : PasoWizard.values() ) {
            menu.addElement( new DefaultMenuItem( paso.getDescripcion() ) );
        }
    }

    /**
     * Comprueba que la pagina en la que estoy es la que debo Si han copiado y pegado la url y no es donde debo estar
     * redirijo a donde quiera. Debe combinarse con la comprobacion en el fichero page.xml de cada pagina Ejemplo:
     * <action execute="#{manejadorWizard.compruebaPaginaActual('DATOS_EXP')}"/>
     * 
	 * Las reglas de navegacion asociadas a estas acciones es recomendable situarlas en el fichero pages.xml
     * @return
     */
    public String compruebaPaginaActual( String idPagina ) {
        if ( !this.pasoActual.name().equals( idPagina ) ) {
            switch ( pasoActual ) {
                case PAGINA1:
                    return "forzarPagina1";
                case PAGINA2:
                    return "forzarPagina2";              
                default:
                    return "forzarPagina1";
            }

        }
        return "";
    }

    public boolean mostrarAnterior() {
        return pasoActual.ordinal() != 0;
    }

    public boolean mostrarSiguiente() {
        return pasoActual.ordinal() != ( PasoWizard.values().length - 1 );
    }
	/**
	* Metodo para indicar si se ha de mostrar habilitado o no el boton Siguiente
	**/
    public boolean habilitarSiguiente() {
        WizardManejable manejador = ( WizardManejable ) Component.getInstance( manejadores.get( pasoActual ), true );
        return manejador.habilitarSiguiente();
    }

    /**
     * Metodo para navegar a la siguiente pantalla. Calcula si estoy en la ultima, en caso contrario llama al metodo
     * siguiente del manejador concreto.
     * 
     * @return
     */
    public String siguiente() {
        WizardManejable manejador = ( WizardManejable ) Component.getInstance( manejadores.get( pasoActual ), true );
        int posicion = pasoActual.ordinal();
        // Si estoy en el ultimo paso me mantengo ahi
        if ( posicion == PasoWizard.values().length - 1 ) {
            log.debug( "No puedo avanzar mas me mantengo en #0", pasoActual.name() );
        }
        // Si estoy en pasos anteriores avanzo
        else {
            log.debug( "Avanzo del paso #0 al paso #1", pasoActual.name(), PasoWizard.values()[posicion + 1].name() );
            this.pasoActual = PasoWizard.values()[posicion + 1];
            return manejador.avanzar();
        }
        return null;
    }

    /**
     * Metodo para navegar a la pantalla anterior
     * 
     * @return
     */
    public String anterior() {
        int posicion = pasoActual.ordinal();
        WizardManejable manejador = ( WizardManejable ) Component.getInstance( manejadores.get( pasoActual ), true );
        // Si estoy en el primer paso me mantengo ahi
        if ( posicion == 0 ) {
            log.debug( "No puedo retroceder me mantengo en #0", pasoActual.name() );
        }
        // Si estoy en pasos posteriores retrocedo
        else {
            log.debug( "Retrocedo del paso #0 al paso #1", pasoActual.name(), PasoWizard.values()[posicion - 1].name() );
            this.pasoActual = PasoWizard.values()[posicion - 1];
            return manejador.retroceder();
        }
        return null;
    }

    /**
     * Metodo para cancelar todo el wizard
     * 
     * @return
     */
    public String cancelar() {
        WizardManejable manejador = ( WizardManejable ) Component.getInstance( manejadores.get( pasoActual ), true );
        return manejador.cancelar();

    }

    public PasoWizard getPasoActual() {
        return pasoActual;
    }

    public void setPasoActual( PasoWizard pasoActual ) {
        this.pasoActual = pasoActual;
    }

    public MenuModel getMenu() {
        return menu;
    }

    public void setMenu( MenuModel menu ) {
        this.menu = menu;
    }

}
